// ============================================================
// Project				:	Pipe Wrench
// Author				:	Iban
// Description			:	Uses a modified Wrench to repair vehicles.
// ============================================================
// Table of Contents
// 1. Datablocks
// 2. Functions
// ============================================================

// ============================================================
// Section 1 : Datablocks
// ============================================================
datablock itemData(pipeWrenchItem : wrenchItem)
{
	category = "Weapon";
	uiName = "Pipe Wrench";
	image = pipeWrenchImage;
	colorShiftColor = "0.750000 0.000000 0.000000 1.000000";
};

datablock shapeBaseImageData(pipeWrenchImage : wrenchImage)
{
	// SpaceCasts
	raycastWeaponRange = 4;
	raycastWeaponTargets = $TypeMasks::All;
	raycastDirectDamage = 0;
	raycastExplosionProjectile = wrenchProjectile;
	
	item = pipeWrenchItem;
	projectile = wrenchProjectile;
	colorShiftColor = "0.750000 0.000000 0.000000 1.000000";
	showBricks = 0;
	
	// State change needed so that the wrench will autofire without reclicking.
	stateTransitionOnTriggerUp[4] = "";
	stateTransitionOnTriggerDown[4] = "StopFire";
};

// ============================================================
// Section 2 : Functions
// ============================================================
function pipeWrenchImage::onPreFire(%this, %obj, %slot)
{
	// Ain't got no fucking idea what the thread name is, so this works.
	wrenchImage.onPreFire(%obj, %slot);
}

function pipeWrenchImage::onFire(%this, %obj, %slot)
{
	if(%this.raycastWeaponRange > 0)
	{
		// Directional Math
		%fvec		= %obj.getForwardVector();
		%fX			= getWord(%fvec,0);
		%fY			= getWord(%fvec,1);
		
		%evec		= %obj.getEyeVector();
		%eX			= getWord(%evec,0);
		%eY			= getWord(%evec,1);
		%eZ			= getWord(%evec,2);
		
		%eXY		= mSqrt((%eX * %eX) + (%eY * %eY));
		%aimVec		= (%fX * %eXY) SPC (%fY * %eXY) SPC %eZ;
		
		// Range
		%range		= %this.raycastWeaponRange * getWord(%obj.getScale(), 2);
		
		// 100 is the maximum effective range of a raycast.
		if(%range > 100)
		{
			%rangeRem = %range - 100;
			%range = 100;
		}
		
		// Raycast Parameters
		%start		= %obj.getEyePoint();
		%end[0]		= vectorAdd(%start, vectorScale(%aimVec, %range)); // [0] required for loop
		%targets	= %this.raycastWeaponTargets;
		
		// Raycasting
		%ray		= ContainerRayCast(%start, %end[0], %targets, %obj);
		%col		= getWord(%ray, 0);
		
		if(isObject(%col))
		{
			%pos	= posFromRaycast(%ray);
			%normal	= normalFromRaycast(%ray);
			%end[0]	= %pos;
			%colclass	= %col.getClassName();
			
			return %this.onHitObject(%obj, %slot, %col, %pos, %normal, %objsHit);
		}
		
		if(%rangeRem > 0)
		{
			for(%a = 1; %a <= 10 && %rangeRem > 0; %a++)
			{
				%range		= %rangeRem;
				
				if(%range > 100)
				{
					%rangeRem = %range - 100;
					%range = 100;
				}
				
				%start[%a]	= %end[%a - 1];
				%end[%a]	= vectorAdd(%start[%a], vectorScale(%aimVec, %range)); 
				
				%ray		= ContainerRayCast(%start[%a], %end[%a], %targets, %obj);
				%col		= getWord(%ray, 0);
				
				if(isObject(%col))
				{
					%pos		= posFromRaycast(%ray);
					%normal		= normalFromRaycast(%ray);
					%end[%a]	= %pos;
					%colclass	= %col.getClassName();
					
					return %this.onHitObject(%obj, %slot, %col, %pos, %normal, %objsHit);
				}
			}
		}
	}
	else
	{
		return Parent::onFire(%this, %obj, %slot);
	}
}

function pipeWrenchImage::onStopFire(%this, %obj, %slot)
{
	%obj.playThread(2, "root");
}

function pipeWrenchImage::onHitObject(%this, %obj, %slot, %col, %pos, %normal)
{
	// Interiors do not have the getType function and will have console spam if you wrench the bedroom.
	// Otherwise, we're checking for a vehicle or a Tank Turret.
	if(isObject(%client = %obj.client) && (%col.getClassName() !$= "InteriorInstance") && (%col.getType() & $TypeMasks::VehicleObjectType || (isObject(TankTurretPlayer) ? %col.getDatablock().getID() == TankTurretPlayer.getID() : 0)))
	{
		if(!isObject(%obj.getObjectMount()))
		{
			%objMini = getMiniGameFromObject(%obj);
			%colMini = getMiniGameFromObject(%col.brickGroup);
			
			// I do not know why Space Guy set the teams 0 based and the vehicle spawn's teams 1 based.
			%vehicleAlliance = %col.spawnBrick.tdmAlliedTeam - 1;
			
			// Both objects are outside of a minigame, or they are both exist and are in the same minigame and their TDM Teams allign...
			if((!isObject(%objMini) && !isObject(%colMini)) || ((isObject(%objMini) && isObject(%colMini)) && %objMini.getID() == %colMini.getID() && (%vehicleAlliance $= "" || %vehicleAlliance < 0 || %client.tdmTeam == %vehicleAlliance)))
			{
				if(%col.getType() & $TypeMasks::VehicleObjectType)
				{
					%type = "Vehicle";
				}
				else if(%col.getDatablock().getID() == TankTurretPlayer.getID())
				{
					%type = "Turret";
				}
				
				%damLvl = %col.getDamageLevel();
				%maxDmg = %col.getDatablock().maxDamage;
				
				// Damaged and Not Destroyed
				if(%damLvl < %maxDmg && %damLvl > 0)
				{
					if(%damLvl < 15)
					{
						%repair = %damLvl;
					}
					else
					{
						%repair = 15;
					}
					
					%col.setDamageLevel(%damLvl - %repair);
					
					%damLvl = %col.getDamageLevel();
					%percent = %col.getDamagePercent() * 100;
					
					if(%damLvl > 0)
					{
						%periodPoint = strPos(%percent, ".");
						
						if(%periodPoint > -1)
						{
							%pureInt = getSubStr(%percent, 0, %periodPoint);
							%trailingStr = getSubStr(%percent, %periodPoint + 1, 2);
							%percentStr = %pureInt @ "." @ %trailingStr;
						}
						else
						{
							%percentStr = %percent;
						}
						
						commandToClient(%client, 'centerPrint', "<color:CCCCCC>" @ %type SPC "Damage:" SPC %percentStr @ "%", 3);
					}
					else
					{
						commandToClient(%client, 'centerPrint', "<color:CCCCCC>" @ %type SPC "Repaired!", 3);
					}
					
					ServerPlay3d(wrenchHitSound, %pos);
				}
				// Not Damaged
				else if(%damLvl == 0)
				{
					ServerPlay3d(wrenchMissSound, %pos);
				}
				// Destroyed
				else if(%damLvl >= %maxDmg)
				{
					commandToClient(%client, 'centerPrint', "<color:CCCCCC>You're going to need a new ride.", 3);
					ServerPlay3d(wrenchMissSound, %pos);
				}
			}
			// If either object are outside a minigame, or they are both inside one but it isn't the same one..
			else if((isObject(%objMini) && !isObject(%colMini)) || (!isObject(%objMini) && isObject(%colMini)) || %objMini.getID() != %colMini.getID())
			{
				commandToClient(%client, 'centerPrint', "You cannot repair items outside of your minigame.", 3);
				ServerPlay3d(wrenchMissSound, %pos);
			}
			// Otherwise, the error was due to TDM.
			else if(%vehicleAlliance !$= "" && %vehicleAlliance >= 0 && %client.tdmTeam != %vehicleAlliance)
			{
				commandToClient(%client, 'centerPrint', "You cannot repair your enemy's vehicles.", 3);
				ServerPlay3d(wrenchMissSound, %pos);
			}
		}
		else
		{
			commandToClient(%client, 'centerPrint', "You must be out of your vehicle to repair things.", 3);
			ServerPlay3d(wrenchMissSound, %pos);
		}
	}
	else
	{
		ServerPlay3d(wrenchMissSound, %pos);
	}
	
	if(isObject(%this.raycastExplosionProjectile))
	{
		%scaleFactor = getWord(%obj.getScale(), 2);
		
		%p = new Projectile()
		{
			dataBlock		= %this.raycastExplosionProjectile;
			initialPosition	= %pos;
			initialVelocity	= %normal;
			sourceObject	= %obj;
			client			= %obj.client;
			sourceSlot		= 0;
			originPoint		= %pos;
		};
		MissionCleanup.add(%p);
		
		%p.setScale(%scaleFactor SPC %scaleFactor SPC %scaleFactor);
		%p.explode();
	}
}
